@@ -76,7 +76,6 @@ static void
 install_dialog_handle_close (InstallDialogHandle *handle)
 {
   g_clear_pointer (&handle->dialog, gtk_window_destroy);
-
   install_dialog_handle_free (handle);
 }
 
@@ -177,6 +176,7 @@ handle_prepare_install (XdpImplDynamicLauncher *object,
                         GVariant               *arg_icon_v,
                         GVariant               *arg_options)
 {
+  g_autoptr(GtkWindowGroup) window_group = NULL;
   g_autoptr(Request) request = NULL;
   const char *sender;
   GdkSurface *surface;
@@ -314,6 +314,9 @@ handle_prepare_install (XdpImplDynamicLauncher *object,
 
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
+  window_group = gtk_window_group_new ();
+  gtk_window_group_add_window (window_group, dialog);
+
   handle->dialog = g_object_ref_sink (dialog);
   handle->entry = entry;
 
