@@ -60,6 +60,7 @@ struct _ScreenCastWidget
   ulong                  monitors_changed_handler_id;
 
   ShellIntrospect       *shell_introspect;
+  GListModel            *filter_model;
   ulong                  windows_changed_handler_id;
 
   uint                   selection_changed_timeout_id;
@@ -70,25 +71,25 @@ struct _ScreenCastWidget
 };
 
 static GQuark quark_monitor_widget_data;
-static GQuark quark_window_widget_data;
 
 G_DEFINE_TYPE (ScreenCastWidget, screen_cast_widget, GTK_TYPE_BOX)
 
+static void schedule_selection_change (ScreenCastWidget *widget);
+
 /*
  * Auxiliary methods
  */
 
 static GtkWidget *
-create_window_widget (Window *window)
+create_window_widget (ShellWindow *window)
 {
   GtkWidget *window_image;
   GtkWidget *check_image;
   GtkWidget *row;
   GIcon *icon = NULL;
   g_autoptr(GDesktopAppInfo) info = NULL;
-  g_autofree char *escaped_name = NULL;
 
-  info = g_desktop_app_info_new (window_get_app_id (window));
+  info = g_desktop_app_info_new (shell_window_get_app_id (window));
   if (info != NULL)
     icon = g_app_info_get_icon (G_APP_INFO (info));
   if (icon == NULL)
@@ -102,14 +103,11 @@ create_window_widget (Window *window)
   row = adw_action_row_new ();
   adw_action_row_add_prefix (ADW_ACTION_ROW (row), window_image);
   adw_action_row_add_suffix (ADW_ACTION_ROW (row), check_image);
+  gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), TRUE);
+  gtk_list_box_row_set_selectable (GTK_LIST_BOX_ROW (row), FALSE);
 
-  escaped_name = g_markup_escape_text (window_get_title (window), -1);
-  adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), escaped_name);
+  g_object_bind_property (window, "title", row, "title", G_BINDING_SYNC_CREATE);
 
-  g_object_set_qdata_full (G_OBJECT (row),
-                           quark_window_widget_data,
-                           window_dup (window),
-                           (GDestroyNotify) window_free);
   g_object_set_data (G_OBJECT (row), "check", check_image);
   return row;
 }
@@ -146,8 +144,6 @@ monitor_container_get_selected_buttons (GtkWidget *monitor_container)
   return selected_monitor_buttons;
 }
 
-static void
-schedule_selection_change (ScreenCastWidget *widget);
 static void
 on_monitor_button_toggled_cb (GtkToggleButton *monitor_button, ScreenCastWidget *self)
 {
@@ -199,54 +195,27 @@ create_monitor_button (Monitor *monitor, const MonitorIllustration *monitor_illu
 }
 
 static gboolean
-should_skip_window (Window    *window,
-                    GtkWindow *toplevel)
+should_show_window (ShellWindow *window,
+                    ScreenCastWidget *self)
 {
+  GtkWidget *toplevel;
   g_autofree char *processed_app_id = NULL;
 
-  if (g_strcmp0 (window_get_title (window),
-                 gtk_window_get_title (toplevel)) != 0)
-    return FALSE;
+  toplevel = gtk_widget_get_ancestor (GTK_WIDGET (self), GTK_TYPE_WINDOW);
 
-  processed_app_id = g_strdup (window_get_app_id (window));
+  if (toplevel && g_strcmp0 (shell_window_get_title (window),
+                             gtk_window_get_title (GTK_WINDOW (toplevel))) != 0)
+    return TRUE;
+
+  processed_app_id = g_strdup (shell_window_get_app_id (window));
   if (g_str_has_suffix (processed_app_id, ".desktop"))
     processed_app_id[strlen (processed_app_id) -
                      strlen (".desktop")] = '\0';
 
   if (g_strcmp0 (processed_app_id, g_get_prgname ()) != 0)
-    return FALSE;
+    return TRUE;
 
-  return TRUE;
-}
-
-static void
-update_windows_list (ScreenCastWidget *widget)
-{
-  GtkListBox *window_list = GTK_LIST_BOX (widget->window_list);
-  GtkWidget *toplevel;
-  GtkWidget *child;
-  GPtrArray *windows;
-
-  while ((child = gtk_widget_get_first_child (GTK_WIDGET (window_list))) != NULL)
-    gtk_list_box_remove (window_list, child);
-
-  toplevel = gtk_widget_get_ancestor (GTK_WIDGET (widget), GTK_TYPE_WINDOW);
-  if (!toplevel)
-    return;
-
-  windows = shell_introspect_get_windows (widget->shell_introspect);
-
-  for (size_t i = 0; windows && i < windows->len; i++)
-    {
-      Window *window = g_ptr_array_index (windows, i);
-      GtkWidget *window_widget;
-
-      if (should_skip_window (window, GTK_WINDOW (toplevel)))
-        continue;
-
-      window_widget = create_window_widget (window);
-      gtk_list_box_append (window_list, window_widget);
-    }
+  return FALSE;
 }
 
 static void
@@ -351,18 +320,35 @@ is_row_selected (GtkListBoxRow *row)
 }
 
 static void
-set_row_is_selected (GtkListBoxRow *row,
-                     gboolean is_selected)
+set_row_is_selected (ScreenCastWidget *widget,
+                     GtkListBoxRow    *row,
+                     gboolean          is_selected)
 {
   GtkWidget *check_image = g_object_get_data (G_OBJECT (row), "check");
   gtk_widget_set_visible (check_image, is_selected);
+  if (is_selected)
+    {
+      gtk_widget_add_css_class (GTK_WIDGET (row), "window-selected");
+      gtk_accessible_update_state (GTK_ACCESSIBLE (row),
+                                   GTK_ACCESSIBLE_STATE_SELECTED, TRUE,
+                                   -1);
+    }
+  else
+    {
+      gtk_widget_remove_css_class (GTK_WIDGET (row), "window-selected");
+      gtk_accessible_reset_state (GTK_ACCESSIBLE (row),
+                                  GTK_ACCESSIBLE_STATE_SELECTED);
+    }
+  g_object_set_data (G_OBJECT (row), "window-selected",
+                     GINT_TO_POINTER (is_selected));
+  schedule_selection_change (widget);
 }
 
 static void
-unselect_row (GtkListBoxRow *row, GtkListBox *listbox)
+unselect_row (GtkListBoxRow    *row,
+              ScreenCastWidget *widget)
 {
-  set_row_is_selected (row, FALSE);
-  gtk_list_box_unselect_row (listbox, row);
+  set_row_is_selected (widget, row, FALSE);
 }
 
 static void
@@ -376,18 +362,11 @@ reset_selection (ScreenCastWidget *self)
 {
 
   foreach_widget_child (self->monitor_container, (GFunc) untoggle_button, self->monitor_container);
-  foreach_widget_child (self->window_list, (GFunc) unselect_row, self->window_list);
+  foreach_widget_child (self->window_list, (GFunc) unselect_row, self);
 
   auto_select_singular_monitor (self);
 }
 
-static void
-on_windows_changed (ShellIntrospect  *shell_introspect,
-                    ScreenCastWidget *widget)
-{
-  update_windows_list (widget);
-}
-
 static void
 on_stack_switch (ScreenCastWidget *self,
                  GParamSpec *pspec,
@@ -405,36 +384,42 @@ on_row_activated (ScreenCastWidget *self,
     return;
 
   if (is_row_selected (row))
-    {
-      set_row_is_selected (row, FALSE);
-      gtk_list_box_unselect_row (box, row);
-    }
+    set_row_is_selected (self, row, FALSE);
   else
-    {
-      set_row_is_selected (row, TRUE);
-      gtk_list_box_select_row (box, row);
-    }
+    set_row_is_selected (self, row, TRUE);
 }
 
-static void
-update_selected_row (GtkListBoxRow *row, gpointer user_data)
+GList *
+get_selected_windows (ScreenCastWidget *widget)
 {
-  set_row_is_selected (row, gtk_list_box_row_is_selected (row));
+  unsigned int i;
+  GList *windows = NULL;
+
+  for (i = 0; i < g_list_model_get_n_items (widget->filter_model); i++)
+    {
+      GtkListBoxRow *row =
+        gtk_list_box_get_row_at_index (GTK_LIST_BOX (widget->window_list), i);
+
+      if (g_object_get_data (G_OBJECT (row), "window-selected"))
+        {
+          windows = g_list_append (windows,
+                                   g_list_model_get_item (widget->filter_model, i));
+        }
+    }
+
+  return windows;
 }
 
 static gboolean
 emit_selection_change_in_idle_cb (ScreenCastWidget *widget)
 {
   g_autoptr(GList) selected_monitor_buttons = NULL;
-  g_autoptr(GList) selected_window_rows = NULL;
-
-  /* Update the selected rows */
-  foreach_widget_child (widget->window_list, (GFunc) update_selected_row, NULL);
+  g_autolist(ShellWindow) selected_windows = NULL;
 
   selected_monitor_buttons = monitor_container_get_selected_buttons (widget->monitor_container);
-  selected_window_rows = gtk_list_box_get_selected_rows (GTK_LIST_BOX (widget->window_list));
+  selected_windows = get_selected_windows (widget);
   g_signal_emit (widget, signals[HAS_SELECTION_CHANGED], 0,
-                 !!selected_monitor_buttons || !!selected_window_rows);
+                 !!selected_monitor_buttons || !!selected_windows);
 
   widget->selection_changed_timeout_id = 0;
   return G_SOURCE_REMOVE;
@@ -450,14 +435,6 @@ schedule_selection_change (ScreenCastWidget *widget)
     g_idle_add (G_SOURCE_FUNC (emit_selection_change_in_idle_cb), widget);
 }
 
-static void
-on_selected_rows_changed (ScreenCastWidget *widget, GtkListBox *box)
-{
-  /* GtkListBox activates rows after selecting them, which prevents
-   * us from emitting the HAS_SELECTION_CHANGED signal here */
-  schedule_selection_change (widget);
-}
-
 static void
 monitors_changed_cb (ScreenCastWidget *self, gconstpointer user_data)
 {
@@ -487,6 +464,8 @@ screen_cast_widget_finalize (GObject *object)
       widget->selection_changed_timeout_id = 0;
     }
 
+  g_clear_object (&widget->filter_model);
+
   g_clear_object (&widget->colors_provider);
 
   G_OBJECT_CLASS (screen_cast_widget_parent_class)->finalize (object);
@@ -516,7 +495,6 @@ screen_cast_widget_root (GtkWidget *widget)
   GTK_WIDGET_CLASS (screen_cast_widget_parent_class)->root (widget);
 
   update_monitor_container (self);
-  update_windows_list (self);
   reset_selection (self);
 }
 
@@ -552,7 +530,6 @@ screen_cast_widget_class_init (ScreenCastWidgetClass *klass)
   gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_list);
 
   quark_monitor_widget_data = g_quark_from_static_string ("-monitor-widget-connector-quark");
-  quark_window_widget_data = g_quark_from_static_string ("-window-widget-connector-quark");
 }
 
 static void
@@ -570,6 +547,9 @@ adjust_to_color_scheme (ScreenCastWidget *self, gpointer user_data)
 static void
 screen_cast_widget_init (ScreenCastWidget *widget)
 {
+  GtkExpression *expression;
+  g_autoptr(GtkFilter) filter = NULL;
+
   widget->selection_changed_timeout_id = 0;
 
   gtk_widget_init_template (GTK_WIDGET (widget));
@@ -594,17 +574,26 @@ screen_cast_widget_init (ScreenCastWidget *widget)
 
   adjust_to_color_scheme (widget, adw_style_manager_get_default ());
 
-  /* Connect windows changed listener */
-  widget->windows_changed_handler_id =
-    g_signal_connect (widget->shell_introspect,
-                      "windows-changed",
-                      G_CALLBACK (on_windows_changed),
-                      widget);
   shell_introspect_ref_listeners (widget->shell_introspect);
 
+  expression = gtk_cclosure_expression_new (G_TYPE_BOOLEAN, NULL, 0,
+                                            NULL,
+                                            G_CALLBACK (should_show_window),
+                                            widget, NULL);
+
+  filter = GTK_FILTER (gtk_bool_filter_new (expression));
+  widget->filter_model = G_LIST_MODEL (gtk_filter_list_model_new (NULL, NULL));
+  gtk_filter_list_model_set_model (GTK_FILTER_LIST_MODEL (widget->filter_model),
+                                   shell_introspect_get_windows (widget->shell_introspect));
+  gtk_filter_list_model_set_filter (GTK_FILTER_LIST_MODEL (widget->filter_model), filter);
+
+  gtk_list_box_bind_model (GTK_LIST_BOX (widget->window_list),
+                           widget->filter_model,
+                           (GtkListBoxCreateWidgetFunc) create_window_widget,
+                           widget, NULL);
+
   g_object_connect (widget->window_list,
                     "swapped-signal::row-activated", G_CALLBACK (on_row_activated), widget,
-                    "swapped-signal::selected-rows-changed", G_CALLBACK (on_selected_rows_changed), widget,
                     NULL);
 
   g_signal_connect_swapped (widget->source_type,
@@ -695,7 +684,7 @@ screen_cast_widget_get_selected_streams (ScreenCastWidget *self)
   ScreenCastStreamInfo *info;
   g_autoptr(GPtrArray) streams = NULL;
   g_autoptr(GList) selected_monitor_buttons = NULL;
-  g_autoptr(GList) selected_window_rows = NULL;
+  g_autolist(ShellWindow) selected_windows = NULL;
   uint32_t id = 0;
   GList *l;
 
@@ -704,10 +693,9 @@ screen_cast_widget_get_selected_streams (ScreenCastWidget *self)
 
   selected_monitor_buttons = monitor_container_get_selected_buttons (self->monitor_container);
 
-  selected_window_rows =
-    gtk_list_box_get_selected_rows (GTK_LIST_BOX (self->window_list));
+  selected_windows = get_selected_windows (self);
 
-  if (!selected_monitor_buttons && !selected_window_rows)
+  if (!selected_monitor_buttons && !selected_windows)
     return g_steal_pointer (&streams);
 
   for (l = selected_monitor_buttons; l; l = l->next)
@@ -734,15 +722,13 @@ screen_cast_widget_get_selected_streams (ScreenCastWidget *self)
         }
     }
 
-  for (l = selected_window_rows; l; l = l->next)
+  for (l = selected_windows; l; l = l->next)
     {
-      Window *window;
-
-      window = g_object_get_qdata (G_OBJECT (l->data), quark_window_widget_data);
+      ShellWindow *window = l->data;
 
       info = g_new0 (ScreenCastStreamInfo, 1);
       info->type = SCREEN_CAST_SOURCE_TYPE_WINDOW;
-      info->data.window = window_dup (window);
+      info->data.window = g_object_ref (window);
       info->id = id++;
       g_ptr_array_add (streams, info);
     }
