@@ -354,10 +354,10 @@ init_settings_table (XdpImplSettings *settings,
     "org.gnome.desktop.calendar",
     "org.gnome.desktop.input-sources",
     "org.gnome.desktop.interface",
+    "org.gnome.desktop.peripherals.mouse",
     "org.gnome.desktop.privacy",
     "org.gnome.desktop.sound",
     "org.gnome.desktop.wm.preferences",
-    "org.gnome.settings-daemon.peripherals.mouse",
     "org.gnome.settings-daemon.plugins.xsettings",
   };
   size_t i;
